#
# This file is part of GNU Enterprise.
#
# GNU Enterprise is free software; you can redistribute it
# and/or modify it under the terms of the GNU General Public
# License as published by the Free Software Foundation; either
# version 2, or(at your option) any later version.
#
# GNU Enterprise is distributed in the hope that it will be
# useful, but WITHOUT ANY WARRANTY; without even the implied
# warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
# PURPOSE. See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public
# License along with program; see the file COPYING. If not,
# write to the Free Software Foundation, Inc., 59 Temple Place
# - Suite 330, Boston, MA 02111-1307, USA.
#
# Copyright 2000-2002 Free Software Foundation
#
# FILE:
# _pgsql/DBdriver.py
#
# DESCRIPTION:
# A core Postgresql dbdriver that the other (specific)
# postgresql drivers can extend
#
# NOTES:
#

import string
from string import lower, join, split
import sys
from gnue.common import GDebug, GDataObjects
from gnue.common.dbdrivers._dbsig.DBdriver \
   import DBSIG_RecordSet, DBSIG_ResultSet, DBSIG_DataObject

from gnue.common.dbdrivers._dbsig.DBdriver import DBSIG_DataObject_Object as PGSQL_DataObject_Object
from gnue.common.dbdrivers._dbsig.DBdriver import DBSIG_DataObject_SQL as PGSQL_DataObject_SQL


class PGSQL_RecordSet(DBSIG_RecordSet):
  def _buildUpdateStatement(self):
    updates = []
    for field in self._modifiedFlags.keys():
##    # To convert date from format '2002-12-31 23:59:59,99'
##    # into '2002-12-31 23:59:59' format.
##    # We have to determine whether given string is date\time
##    # maybe it's the most stupid way, but it should work.
##    # TODO: if in ANY other field data of this format and comma in
##    # place will exist, the remaining string from first ',' will be
##    # eaten.
##    # If you know better decision - please, modify this code.
##
## This should have been submitted as a patch for peer review.
## For starters, it can break non-date fields w/commas.
## Secondly, such checks belong in _toSqlString!
## -- jcater
##
##      tmpDate = self._fields[field]
##      if ((len(tmpDate)==22) and (tmpDate[-3]==',')):
##        tmpDate = tmpDate.split(',')
##        tmpDate = tmpDate[0]
##      updates.append ("%s=%s" % (field,
##        self._parent._dataObject._toSqlString(tmpDate)))

      updates.append ("%s=%s" % (field,
        self._parent._dataObject._toSqlString(self._fields[field])))

    if self._parent._dataObject._primaryIdField:
      where = [self._parent._dataObject._primaryIdFormat % \
          self._initialData[self._parent._dataObject._primaryIdField]  ]
    else:
      where = []
      for field in self._initialData.keys():
        if self._initialData[field] == None:
          where.append ("%s IS NULL" % field)
        else:
          where.append ("%s=%s" % (field, self._parent._dataObject._toSqlString(self._initialData[field])))
    return "UPDATE %s SET %s WHERE %s" % \
       (self._parent._dataObject.table, string.join(updates,','), \
        string.join(where,' AND ') )


class PGSQL_ResultSet(DBSIG_ResultSet):
  def __init__(self, dataObject, cursor=None, defaultValues={}, masterRecordSet=None):
    DBSIG_ResultSet.__init__(self, dataObject, \
                             cursor, defaultValues, masterRecordSet)
    self._recordSetClass = PGSQL_RecordSet

class PGSQL_DataObject(DBSIG_DataObject):

  schema2nativeTypes={"auto":"int4",
                      "int":"int",
                      "number":"int",
                      "float":"float",
                      "decimal":"decimal",
                      "varchar":"varchar",
                      "char":"char",
                      "blob":"text",
                      "text":"text",
                      "longtext":"text",
                      "date":"date",
                      "datetime":"datetime",
                      "timestamp":"timestamp"}

  def __init__(self, pgdriver=None, pgresultset=None):
    DBSIG_DataObject.__init__(self)
    self._connectString = 'host=%s dbname=%s user=%s password=%s port=%s'
    self._escapeSingleQuote = '\\'
    # date/time format
    self._dateTimeFormat = "'%Y-%m-%d %H:%M:%S'"

    if pgdriver:
      self._pgdriver = pgdriver
      self._DatabaseError = self._pgdriver.DatabaseError
    if pgresultset:
      self._resultSetClass = pgresultset

  def connect(self, connectData={}):
    GDebug.printMesg(1,"Postgresql database driver initializing")
    try:
      try:
        port = connectData['port']
      except:
        port  = '5432'

      self._dataConnection = self._pgdriver.connect(self._connectString %
                                                    (connectData['host'],
                                                     connectData['dbname'],
                                                     connectData['_username'],
                                                     connectData['_password'],
                                                     port))
      self.triggerExtensions = PGSQL_TriggerExtensions(self._dataConnection)
    except self._DatabaseError, value:
      GDebug.printMesg(1,"Connect String: %s" % (self._connectString % \
             (connectData['host'],
              connectData['dbname'],
              connectData['_username'],
              connectData['_password'],
              port)))
      GDebug.printMesg(1,"Exception %s " % value)
      raise GDataObjects.LoginError, value

    try:
      encoding = ""
      try:
        encoding = connectData['encoding']
      except KeyError:
        # if encoding is not defined in connectData use gnue.conf setting instead
        try:
          encoding = gConfig('encoding')
        except:
          pass

      if encoding!="":
        GDebug.printMesg(1,'Setting postgresql client_encoding to %s' % encoding)
        cursor = self._dataConnection.cursor()
        cursor.execute("SET CLIENT_ENCODING TO '%s'" % encoding)
        cursor.close()

    except self._DatabaseError:
      try:
        cursor.close()
      except:
        pass

    if connectData.has_key('datetimeformat'):
      self._dateTimeFormat = "'%s'" % connectData['datetimeformat']


    self._postConnect()

  def _postConnect(self):
    self.triggerExtensions = PGSQL_TriggerExtensions(self._dataConnection)

  #
  # Schema (metadata) functions
  #

  # Return a list of the types of Schema objects this driver provides
  def getSchemaTypes(self):
    return [('view','View',1), ('table','Table',1)]

  # Return a list of Schema objects
  def getSchemaList(self, type=None):
    includeTables = (type in ('table','sources', None))
    includeViews = (type in ('view','sources', None))

    inClause = []
    if includeTables:
      inClause.append ("'r'")
    if includeViews:
      inClause.append ("'v'")

    # TODO: This excludes any system tables and views. Should it?
    statement = "select relname, relkind, oid from pg_class " + \
            "where relkind in (%s) " % (join(inClause,',')) + \
            "and relname not like 'pg_%' " + \
            "order by relname"

    cursor = self._dataConnection.cursor()
    cursor.execute(statement)

    list = []
    for rs in cursor.fetchall():
      list.append(GDataObjects.Schema(attrs={'id':rs[2], 'name':rs[0],
                         'type':rs[1] == 'v' and 'view' or 'table',
                         'primarykey': self.__getPrimaryKey(cursor, rs[2])},
                         getChildSchema=self.__getFieldSchema))

    cursor.close()
    return list


  # Find a schema object with specified name
  def getSchemaByName(self, name, type=None):
    statement = "select relname, relkind, oid from pg_class " + \
            "where relname = '%s'" % (name)

    cursor = self._dataConnection.cursor()
    cursor.execute(statement)

    rs = cursor.fetchone()
    if rs:
      schema = GDataObjects.Schema(attrs={'id':rs[2], 'name':rs[0],
                           'type':rs[1] == 'v' and 'view' or 'table',
                           'primarykey': self.__getPrimaryKey(cursor, rs[2]) },
                           getChildSchema=self.__getFieldSchema)
    else:
      schema = None

    cursor.close()
    return schema

  def __getPrimaryKey(self, cursor, oid):
    cursor = self._dataConnection.cursor()
    cursor.execute("select indkey from pg_index where indrelid=%d" % oid)
    rs = cursor.fetchone()
    statement = "select attname from pg_attribute " \
                "where attrelid = %d and attnum = %%d" % oid
    if rs:
      pks = []
      for indpos in string.split(rs[0]):
        cursor.execute(statement % int(indpos))
        pks.append(cursor.fetchone()[0])
      cursor.close()
      return tuple(pks)
    else:
      cursor.close()
      return None

  # Get fields for a table
  def __getFieldSchema(self, parent):

    statement = "select attname, pg_type.oid, typname, " + \
            " attnotnull, atthasdef, atttypmod, attnum, attlen " + \
            "from pg_attribute, pg_type " + \
            "where attrelid = %d and " % (parent.id) + \
            "pg_type.oid = atttypid and attnum >= 0" + \
            "order by attnum"

    cursor = self._dataConnection.cursor()
    cursor.execute(statement)

    list = []
    for rs in cursor.fetchall():

      attrs={'id': rs[1], 'name': rs[0],
             'type':'field', 'nativetype': rs[2],
             'required': rs[3] and not rs[4]}

      if rs[2] in ('numeric','float4','float8','money','bool','int8','int2','int4'):
        attrs['datatype']='number'
      elif rs[2] in ('date','time','timestamp','abstime','reltime'):
        attrs['datatype']='date'
      else:
        attrs['datatype']='text'

      if rs[7] > 0:
        attrs['length'] = rs[7]
      else:
        attrs['length'] = rs[5] - 4


      # Find any default values
      if rs[4]:
        cursor.execute("select adsrc " + \
                       "from pg_attrdef " + \
                       "where adrelid = %d and adnum = %d" % (parent.id, rs[6]))
        defrs = cursor.fetchone()
        if defrs:
          dflt = defrs[0]
          if dflt[:8] == 'nextval(':
            attrs['defaulttype'] = 'sequence'
            attrs['defaultval'] = split(dflt,"'")[1]
          elif dflt == 'now()':
            attrs['defaulttype'] = 'system'
            attrs['defaultval'] = 'timestamp'
          else:
            attrs['defaulttype'] = 'constant'
            attrs['defaultval'] = dflt

      list.append(GDataObjects.Schema(attrs=attrs))

    cursor.close()
    return list
   
#
#  Extensions to Trigger Namespaces
#
class PGSQL_TriggerExtensions:

  def __init__(self, connection):
    self.__connection = connection

  # Return the current date, according to database
  def getTimeStamp(self):
    return self.__singleQuery("select current_timestamp")

  # Return a sequence number from sequence 'name'
  def getSequence(self, name):
    return self.__singleQuery("select nextval('%s')" % name)

  # Run the SQL statement 'statement'
  def sql(self, statement):
    cursor = self.__connection.cursor()
    try:
      cursor.execute(statement)
      cursor.close()
    except:
      cursor.close()
      raise

  # Used internally
  def __singleQuery(self, statement):
    cursor = self.__connection.cursor()
    try:
      cursor.execute(statement)
      rv = cursor.fetchone()
      cursor.close()
    except:
      print "DBdriver.py", "You've got your bar in my foo! And you've got your foo on my bar!  Two great reams that ream well together!"
      GDebug.printMesg(1,"**** Unable to execute extension query")
      GDebug.printMesg(1,"**** %s" % sys.exc_info()[1])
      cursor.close()
      return None

    try:
      return rv[0]
    except:
      return None



